#!/usr/bin/perl 
#

###########################################################
# Add Source and licence info into PNG, XPM, and SVG files
#
#
# Usage: 
# 	add_metadata.pl "[LICENCE]" "[SOURCE]" "[SOURCE URL]" [DIRECTORY]
#
# Example: 
# 	add_metadata.pl "CC-BY-3.0" "Happy Icons" "http://www.happy.com" icons/
#
# Example (without URL):
# 	add_metadata.pl "CC-BY-3.0" "Happy Icons" "" icons/
#

$license = $ARGV[0];
$source = $ARGV[1];
$source_url = $ARGV[2];
$dir_name = $ARGV[3];


use File::Find;
use Cwd;
use SVG::Metadata; 
use XML::Twig;     # Needed if SVG::Metadata fails, to get XML twig to attach new RDF to.

sub process_svg_file {
  for $f (@_) {
	  # print "   Processing file\n";
    my $meta = new SVG::Metadata;
    my $p; eval { $p = $meta->parse($f, retain_xml => 1) };
    if ($p) {
		# print "   Modifying current metadata\n";
      # We merge the image's own metadata with the propagated
      # metadata.  The file's own metadata, where present, take
      # precedence, but keywords are used from both sources.
      eval " \$meta->$_(\$mainmeta->$_()) unless \$meta->$_() "
        for qw(title description publisher publisher_url creator
               creator_url owner owner_url license license_date
               about_url date language);
	  my $added_source = '';
	  if( $meta->publisher() ) {
		  $added_source = $meta->publisher() .  ", Source: $source";
	  }else{
		  $added_source = "Source: $source";
	  }
	  $meta->publisher($added_source);
	  if ( not $meta->publisher_url() ) {
		$meta->publisher_url($source_url)
	  }
	  if( $meta->license() ) {
		  $meta->license($license)
	  }
      my @k = $meta->keywords();
      $meta->removeKeyword('unsorted') if @k > 1;
      writemetatofile($meta, $f);
    } else {
		# print "   No metadata, adding now\n";
      # The image has no metadata of its own, so give it the
      # propagated metadata unaltered:
      writemetatofile($mainmeta, $f);
    }
  }
}



sub get_xml {
  my ($file) = @_;
    my $twig = XML::Twig->new( map_xmlns => {
              'http://web.resource.org/cc/' => "cc",
              'http://www.w3.org/1999/02/22-rdf-syntax-ns#' => "rdf",
              'http://purl.org/dc/elements/1.1/' => "dc",
              'http://www.w3.org/2000/svg' => "svg",
       },
       pretty_print => 'indented',
       comments     => 'keep',
       pi           => 'keep',
       keep_original_prefix => 1,
    );
  eval { $twig->parsefile($file); };
  if ($@) {
    warn "XML::Twig hates $file: $@"; return;
  }
  return $twig;
}


sub writemetatofile {
  my ($meta, $file, $svg) = @_;
  $svg ||= $meta->to_svg();
  if (not $svg) {
    # warn "Processing $file\n";
    my $xml = get_xml($file);
    #my $mdx = get_xml('metadata.rdf');
    my $mdx = XML::Twig->new(map_xmlns => {
                  'http://web.resource.org/cc/' => "cc",
                  'http://www.w3.org/1999/02/22-rdf-syntax-ns#' => "rdf",
                  'http://purl.org/dc/elements/1.1/' => "dc",
        },
        pretty_print => 'indented',
    );
    $mdx->parse($meta->to_rdf());
    if (ref $xml and ref $mdx) {
      $mdx->root()->copy();
      $mdx->root()->paste( first_child => $xml->root());
      open SVGIN, '<', $file; local $/ = '<svg'; my $raw = <SVGIN>; close SVGIN;
      my ($decl) = $raw =~ /(.*?)(<svg|<!-- Created|<!-- Generator|$)/is; # an inexact science
      my $newsvg = $decl . ($xml->root()->sprint());
      if ($newsvg) {
        writemetatofile($meta, $file, $newsvg);
      } else {
        warn "Processing failed on $file"; return;
      }
    } else {
      warn "Cannot process $file"; return;
    }
  } elsif (open SVG, '>:encoding(utf8)', $file) {
	print SVG $svg; close SVG;
  } else { warn "cannot write $file: $!\n"; }
}

sub process_file {
    my $file_name=$_;
	my $full_name = $File::Find::name;
	
	if ( $full_name !~ /^\// ) {
		$full_name = $current_dir . "/" . $full_name;
	}

	if ( !( -d $full_name) ) {
		if ( $file_name =~ /\.png$/i ) {
			&png_file($file_name, $full_name);
		} elsif ( $file_name =~ /\.xpm$/i ) {
			&xpm_file($file_name, $full_name);
		} elsif ( $file_name =~ /\.svg$/i ) {
			&svg_file($file_name, $full_name);
		} else {
			print "X";
		}
	}
}


sub png_file {
	my $file_name = $_[0];
	my $full_name = $_[1];
	# print "PNG: $file_name\n";
	# print "   Adding Source and License info\n";
	print "p";
	# print "mogrify -set License \"$license\" -set Source \"$source\" $full_name \n";
	system("mogrify -set License \"$license\" -set Source \"$source\" -set Source_URL \"$source_url\" \"$full_name\"");
}


sub xpm_file {
	my $file_name = $_[0];
	my $full_name = $_[1];
	# print "XPM: $file_name\n";
	# print "   Adding Source and License info\n";
	print "x";
	if (open(FILE, $full_name)){
		my @lines = <FILE>;
		close(FILE);
		print "-";
		if(open(FILE, ">$full_name")) {
			my $first = 1;
			print "+";
			foreach $line (@lines) {
				print FILE $line;
				# print $line;
				if($first == 1){
					print FILE "/* License: $license	*/\n";
					print FILE "/* Source:  $source		*/\n";
					print FILE "/* Source URL:  $source_url		*/\n";
					# print "/* License: $license	*/\n";
					# print "/* Source:  $source		*/\n";
					$first = 0;
				}
			}
			close FILE;
		} else {
			print "ERROR: could not write file\n";
		}
	} else {
		print "ERROR: file could not be opened\n";
	}
}


sub svg_file {
	my $file_name = $_[0];
	my $full_name = $_[1];
	# print "SVG: $file_name\n";
	print "s";
	process_svg_file($full_name);
	# my $svgmeta = new SVG::Metadata;
	# print "  licence = " . $svgmeta->license() . "\n";
	# print "  publisher = " . $svgmeta->publisher() . "\n";
	# $new = $full_name;
	# $new =~ s/\.svgz$/.svg.gz/;
	# print "$full_name\n";
	# print "$new\n";
	# rename($full_name,$new);
	# system("gunzip $new");
}


# Check input
if( $license eq '' ){
	print "Input ARGS: LICENSE SOURCE SOURCE_URL DIRECTORY\n";
	exit;
} elsif( !( -d $dir_name ) ){
	print "The 4th input ARG is suposta be a directory\n$dir_name\n";
	print "$license\n$source\n$source_url\n$dir_name";
	exit
}


#################################################################
# Main Code
#
#

$current_dir = cwd();
$mainmeta = new SVG::Metadata;
$mainmeta->publisher("Source: $source");
$mainmeta->publisher_url("$source_url");
$mainmeta->license($license);
print "Searching for xpm, png, svg, and svgz files in $dir_name\n";
find(\&process_file, $dir_name);

print "\n";
